@@ -43,7 +43,7 @@ package Time::OlsonTZ::Data;
 use warnings;
 use strict;
 
-our $VERSION = "0.201407";
+our $VERSION = "0.201702";
 
 use parent "Exporter";
 our @EXPORT_OK = qw(
@@ -83,7 +83,7 @@ retain this format in the future.
 
 =cut
 
-use constant olson_version => "2014g";
+use constant olson_version => "2017b";
 
 =item olson_code_version
 
@@ -92,7 +92,7 @@ module encapsulates.
 
 =cut
 
-use constant olson_code_version => "2014g";
+use constant olson_code_version => "2017b";
 
 =item olson_data_version
 
@@ -101,7 +101,7 @@ module encapsulates.
 
 =cut
 
-use constant olson_data_version => "2014g";
+use constant olson_data_version => "2017b";
 
 =back
 
@@ -120,15 +120,11 @@ values are all C<undef>.
 =cut
 
 my $cn = q(+{ map { ($_ => undef) } qw(
-	Africa/Abidjan Africa/Accra Africa/Addis_Ababa Africa/Algiers
-	Africa/Asmara Africa/Bissau Africa/Blantyre Africa/Bujumbura
-	Africa/Cairo Africa/Casablanca Africa/Ceuta Africa/Dar_es_Salaam
-	Africa/Djibouti Africa/El_Aaiun Africa/Gaborone Africa/Harare
-	Africa/Johannesburg Africa/Kampala Africa/Khartoum Africa/Kigali
-	Africa/Lagos Africa/Lubumbashi Africa/Lusaka Africa/Maputo Africa/Maseru
-	Africa/Mbabane Africa/Mogadishu Africa/Monrovia Africa/Nairobi
-	Africa/Ndjamena Africa/Tripoli Africa/Tunis Africa/Windhoek America/Adak
-	America/Anchorage America/Antigua America/Araguaina
+	Africa/Abidjan Africa/Accra Africa/Algiers Africa/Bissau Africa/Cairo
+	Africa/Casablanca Africa/Ceuta Africa/El_Aaiun Africa/Johannesburg
+	Africa/Khartoum Africa/Lagos Africa/Maputo Africa/Monrovia
+	Africa/Nairobi Africa/Ndjamena Africa/Tripoli Africa/Tunis
+	Africa/Windhoek America/Adak America/Anchorage America/Araguaina
 	America/Argentina/Buenos_Aires America/Argentina/Catamarca
 	America/Argentina/Cordoba America/Argentina/Jujuy
 	America/Argentina/La_Rioja America/Argentina/Mendoza
@@ -138,15 +134,15 @@ my $cn = q(+{ map { ($_ => undef) } qw(
 	America/Atikokan America/Bahia America/Bahia_Banderas America/Barbados
 	America/Belem America/Belize America/Blanc-Sablon America/Boa_Vista
 	America/Bogota America/Boise America/Cambridge_Bay America/Campo_Grande
-	America/Cancun America/Caracas America/Cayenne America/Cayman
-	America/Chicago America/Chihuahua America/Costa_Rica America/Creston
-	America/Cuiaba America/Curacao America/Danmarkshavn America/Dawson
-	America/Dawson_Creek America/Denver America/Detroit America/Edmonton
-	America/Eirunepe America/El_Salvador America/Fortaleza America/Glace_Bay
-	America/Godthab America/Goose_Bay America/Grand_Turk America/Guatemala
-	America/Guayaquil America/Guyana America/Halifax America/Havana
-	America/Hermosillo America/Indiana/Indianapolis America/Indiana/Knox
-	America/Indiana/Marengo America/Indiana/Petersburg
+	America/Cancun America/Caracas America/Cayenne America/Chicago
+	America/Chihuahua America/Costa_Rica America/Creston America/Cuiaba
+	America/Curacao America/Danmarkshavn America/Dawson America/Dawson_Creek
+	America/Denver America/Detroit America/Edmonton America/Eirunepe
+	America/El_Salvador America/Fort_Nelson America/Fortaleza
+	America/Glace_Bay America/Godthab America/Goose_Bay America/Grand_Turk
+	America/Guatemala America/Guayaquil America/Guyana America/Halifax
+	America/Havana America/Hermosillo America/Indiana/Indianapolis
+	America/Indiana/Knox America/Indiana/Marengo America/Indiana/Petersburg
 	America/Indiana/Tell_City America/Indiana/Vevay
 	America/Indiana/Vincennes America/Indiana/Winamac America/Inuvik
 	America/Iqaluit America/Jamaica America/Juneau
@@ -155,37 +151,36 @@ my $cn = q(+{ map { ($_ => undef) } qw(
 	America/Manaus America/Martinique America/Matamoros America/Mazatlan
 	America/Menominee America/Merida America/Metlakatla America/Mexico_City
 	America/Miquelon America/Moncton America/Monterrey America/Montevideo
-	America/Montreal America/Nassau America/New_York America/Nipigon
-	America/Nome America/Noronha America/North_Dakota/Beulah
-	America/North_Dakota/Center America/North_Dakota/New_Salem
-	America/Ojinaga America/Panama America/Pangnirtung America/Paramaribo
-	America/Phoenix America/Port-au-Prince America/Port_of_Spain
-	America/Porto_Velho America/Puerto_Rico America/Rainy_River
+	America/Nassau America/New_York America/Nipigon America/Nome
+	America/Noronha America/North_Dakota/Beulah America/North_Dakota/Center
+	America/North_Dakota/New_Salem America/Ojinaga America/Panama
+	America/Pangnirtung America/Paramaribo America/Phoenix
+	America/Port-au-Prince America/Port_of_Spain America/Porto_Velho
+	America/Puerto_Rico America/Punta_Arenas America/Rainy_River
 	America/Rankin_Inlet America/Recife America/Regina America/Resolute
-	America/Rio_Branco America/Santa_Isabel America/Santarem
-	America/Santiago America/Santo_Domingo America/Sao_Paulo
-	America/Scoresbysund America/Sitka America/St_Johns
-	America/Swift_Current America/Tegucigalpa America/Thule
-	America/Thunder_Bay America/Tijuana America/Toronto America/Vancouver
-	America/Whitehorse America/Winnipeg America/Yakutat America/Yellowknife
-	Antarctica/Casey Antarctica/Davis Antarctica/DumontDUrville
-	Antarctica/Macquarie Antarctica/Mawson Antarctica/Palmer
-	Antarctica/Rothera Antarctica/Syowa Antarctica/Troll Antarctica/Vostok
-	Asia/Aden Asia/Almaty Asia/Amman Asia/Anadyr Asia/Aqtau Asia/Aqtobe
-	Asia/Ashgabat Asia/Baghdad Asia/Bahrain Asia/Baku Asia/Bangkok
-	Asia/Beirut Asia/Bishkek Asia/Brunei Asia/Chita Asia/Choibalsan
-	Asia/Colombo Asia/Damascus Asia/Dhaka Asia/Dili Asia/Dubai Asia/Dushanbe
-	Asia/Gaza Asia/Hebron Asia/Ho_Chi_Minh Asia/Hong_Kong Asia/Hovd
-	Asia/Irkutsk Asia/Jakarta Asia/Jayapura Asia/Jerusalem Asia/Kabul
-	Asia/Kamchatka Asia/Karachi Asia/Kathmandu Asia/Khandyga Asia/Kolkata
-	Asia/Krasnoyarsk Asia/Kuala_Lumpur Asia/Kuching Asia/Kuwait Asia/Macau
-	Asia/Magadan Asia/Makassar Asia/Manila Asia/Muscat Asia/Nicosia
-	Asia/Novokuznetsk Asia/Novosibirsk Asia/Omsk Asia/Oral Asia/Phnom_Penh
-	Asia/Pontianak Asia/Pyongyang Asia/Qatar Asia/Qyzylorda Asia/Rangoon
+	America/Rio_Branco America/Santarem America/Santiago
+	America/Santo_Domingo America/Sao_Paulo America/Scoresbysund
+	America/Sitka America/St_Johns America/Swift_Current America/Tegucigalpa
+	America/Thule America/Thunder_Bay America/Tijuana America/Toronto
+	America/Vancouver America/Whitehorse America/Winnipeg America/Yakutat
+	America/Yellowknife Antarctica/Casey Antarctica/Davis
+	Antarctica/DumontDUrville Antarctica/Macquarie Antarctica/Mawson
+	Antarctica/Palmer Antarctica/Rothera Antarctica/Syowa Antarctica/Troll
+	Antarctica/Vostok Asia/Almaty Asia/Amman Asia/Anadyr Asia/Aqtau
+	Asia/Aqtobe Asia/Ashgabat Asia/Atyrau Asia/Baghdad Asia/Baku
+	Asia/Bangkok Asia/Barnaul Asia/Beirut Asia/Bishkek Asia/Brunei
+	Asia/Chita Asia/Choibalsan Asia/Colombo Asia/Damascus Asia/Dhaka
+	Asia/Dili Asia/Dubai Asia/Dushanbe Asia/Famagusta Asia/Gaza Asia/Hebron
+	Asia/Ho_Chi_Minh Asia/Hong_Kong Asia/Hovd Asia/Irkutsk Asia/Jakarta
+	Asia/Jayapura Asia/Jerusalem Asia/Kabul Asia/Kamchatka Asia/Karachi
+	Asia/Kathmandu Asia/Khandyga Asia/Kolkata Asia/Krasnoyarsk
+	Asia/Kuala_Lumpur Asia/Kuching Asia/Macau Asia/Magadan Asia/Makassar
+	Asia/Manila Asia/Nicosia Asia/Novokuznetsk Asia/Novosibirsk Asia/Omsk
+	Asia/Oral Asia/Pontianak Asia/Pyongyang Asia/Qatar Asia/Qyzylorda
 	Asia/Riyadh Asia/Sakhalin Asia/Samarkand Asia/Seoul Asia/Shanghai
 	Asia/Singapore Asia/Srednekolymsk Asia/Taipei Asia/Tashkent Asia/Tbilisi
-	Asia/Tehran Asia/Thimphu Asia/Tokyo Asia/Ulaanbaatar Asia/Urumqi
-	Asia/Ust-Nera Asia/Vientiane Asia/Vladivostok Asia/Yakutsk
+	Asia/Tehran Asia/Thimphu Asia/Tokyo Asia/Tomsk Asia/Ulaanbaatar
+	Asia/Urumqi Asia/Ust-Nera Asia/Vladivostok Asia/Yakutsk Asia/Yangon
 	Asia/Yekaterinburg Asia/Yerevan Atlantic/Azores Atlantic/Bermuda
 	Atlantic/Canary Atlantic/Cape_Verde Atlantic/Faroe Atlantic/Madeira
 	Atlantic/Reykjavik Atlantic/South_Georgia Atlantic/Stanley
@@ -197,27 +192,27 @@ my $cn = q(+{ map { ($_ => undef) } qw(
 	Etc/GMT+5 Etc/GMT+6 Etc/GMT+7 Etc/GMT+8 Etc/GMT+9 Etc/GMT-1 Etc/GMT-10
 	Etc/GMT-11 Etc/GMT-12 Etc/GMT-13 Etc/GMT-14 Etc/GMT-2 Etc/GMT-3
 	Etc/GMT-4 Etc/GMT-5 Etc/GMT-6 Etc/GMT-7 Etc/GMT-8 Etc/GMT-9 Etc/UCT
-	Etc/UTC Europe/Amsterdam Europe/Andorra Europe/Athens Europe/Belgrade
-	Europe/Berlin Europe/Brussels Europe/Bucharest Europe/Budapest
-	Europe/Chisinau Europe/Copenhagen Europe/Dublin Europe/Gibraltar
-	Europe/Helsinki Europe/Istanbul Europe/Kaliningrad Europe/Kiev
-	Europe/Lisbon Europe/London Europe/Luxembourg Europe/Madrid Europe/Malta
-	Europe/Minsk Europe/Monaco Europe/Moscow Europe/Oslo Europe/Paris
-	Europe/Prague Europe/Riga Europe/Rome Europe/Samara Europe/Simferopol
-	Europe/Sofia Europe/Stockholm Europe/Tallinn Europe/Tirane
+	Etc/UTC Europe/Amsterdam Europe/Andorra Europe/Astrakhan Europe/Athens
+	Europe/Belgrade Europe/Berlin Europe/Brussels Europe/Bucharest
+	Europe/Budapest Europe/Chisinau Europe/Copenhagen Europe/Dublin
+	Europe/Gibraltar Europe/Helsinki Europe/Istanbul Europe/Kaliningrad
+	Europe/Kiev Europe/Kirov Europe/Lisbon Europe/London Europe/Luxembourg
+	Europe/Madrid Europe/Malta Europe/Minsk Europe/Monaco Europe/Moscow
+	Europe/Oslo Europe/Paris Europe/Prague Europe/Riga Europe/Rome
+	Europe/Samara Europe/Saratov Europe/Simferopol Europe/Sofia
+	Europe/Stockholm Europe/Tallinn Europe/Tirane Europe/Ulyanovsk
 	Europe/Uzhgorod Europe/Vienna Europe/Vilnius Europe/Volgograd
-	Europe/Warsaw Europe/Zaporozhye Europe/Zurich Factory HST
-	Indian/Antananarivo Indian/Chagos Indian/Christmas Indian/Cocos
-	Indian/Comoro Indian/Kerguelen Indian/Mahe Indian/Maldives
-	Indian/Mauritius Indian/Mayotte Indian/Reunion MET MST MST7MDT PST8PDT
-	Pacific/Apia Pacific/Auckland Pacific/Chatham Pacific/Chuuk
-	Pacific/Easter Pacific/Efate Pacific/Enderbury Pacific/Fakaofo
-	Pacific/Fiji Pacific/Funafuti Pacific/Galapagos Pacific/Gambier
-	Pacific/Guadalcanal Pacific/Guam Pacific/Honolulu Pacific/Kiritimati
-	Pacific/Kosrae Pacific/Kwajalein Pacific/Majuro Pacific/Marquesas
-	Pacific/Midway Pacific/Nauru Pacific/Niue Pacific/Norfolk Pacific/Noumea
-	Pacific/Pago_Pago Pacific/Palau Pacific/Pitcairn Pacific/Pohnpei
-	Pacific/Port_Moresby Pacific/Rarotonga Pacific/Saipan Pacific/Tahiti
+	Europe/Warsaw Europe/Zaporozhye Europe/Zurich Factory HST Indian/Chagos
+	Indian/Christmas Indian/Cocos Indian/Kerguelen Indian/Mahe
+	Indian/Maldives Indian/Mauritius Indian/Reunion MET MST MST7MDT PST8PDT
+	Pacific/Apia Pacific/Auckland Pacific/Bougainville Pacific/Chatham
+	Pacific/Chuuk Pacific/Easter Pacific/Efate Pacific/Enderbury
+	Pacific/Fakaofo Pacific/Fiji Pacific/Funafuti Pacific/Galapagos
+	Pacific/Gambier Pacific/Guadalcanal Pacific/Guam Pacific/Honolulu
+	Pacific/Kiritimati Pacific/Kosrae Pacific/Kwajalein Pacific/Majuro
+	Pacific/Marquesas Pacific/Nauru Pacific/Niue Pacific/Norfolk
+	Pacific/Noumea Pacific/Pago_Pago Pacific/Palau Pacific/Pitcairn
+	Pacific/Pohnpei Pacific/Port_Moresby Pacific/Rarotonga Pacific/Tahiti
 	Pacific/Tarawa Pacific/Tongatapu Pacific/Wake Pacific/Wallis WET
 ) });
 sub olson_canonical_names() {
@@ -270,21 +265,36 @@ be found in the L</olson_canonical_names> hash.
 =cut
 
 my $li = q(+{
-	"Africa/Asmera" => "Africa/Asmara",
+	"Africa/Addis_Ababa" => "Africa/Nairobi",
+	"Africa/Asmara" => "Africa/Nairobi",
+	"Africa/Asmera" => "Africa/Nairobi",
 	"Africa/Bamako" => "Africa/Abidjan",
 	"Africa/Bangui" => "Africa/Lagos",
 	"Africa/Banjul" => "Africa/Abidjan",
+	"Africa/Blantyre" => "Africa/Maputo",
 	"Africa/Brazzaville" => "Africa/Lagos",
+	"Africa/Bujumbura" => "Africa/Maputo",
 	"Africa/Conakry" => "Africa/Abidjan",
 	"Africa/Dakar" => "Africa/Abidjan",
+	"Africa/Dar_es_Salaam" => "Africa/Nairobi",
+	"Africa/Djibouti" => "Africa/Nairobi",
 	"Africa/Douala" => "Africa/Lagos",
 	"Africa/Freetown" => "Africa/Abidjan",
+	"Africa/Gaborone" => "Africa/Maputo",
+	"Africa/Harare" => "Africa/Maputo",
 	"Africa/Juba" => "Africa/Khartoum",
+	"Africa/Kampala" => "Africa/Nairobi",
+	"Africa/Kigali" => "Africa/Maputo",
 	"Africa/Kinshasa" => "Africa/Lagos",
 	"Africa/Libreville" => "Africa/Lagos",
 	"Africa/Lome" => "Africa/Abidjan",
 	"Africa/Luanda" => "Africa/Lagos",
+	"Africa/Lubumbashi" => "Africa/Maputo",
+	"Africa/Lusaka" => "Africa/Maputo",
 	"Africa/Malabo" => "Africa/Lagos",
+	"Africa/Maseru" => "Africa/Johannesburg",
+	"Africa/Mbabane" => "Africa/Johannesburg",
+	"Africa/Mogadishu" => "Africa/Nairobi",
 	"Africa/Niamey" => "Africa/Lagos",
 	"Africa/Nouakchott" => "Africa/Abidjan",
 	"Africa/Ouagadougou" => "Africa/Abidjan",
@@ -292,11 +302,13 @@ my $li = q(+{
 	"Africa/Sao_Tome" => "Africa/Abidjan",
 	"Africa/Timbuktu" => "Africa/Abidjan",
 	"America/Anguilla" => "America/Port_of_Spain",
+	"America/Antigua" => "America/Port_of_Spain",
 	"America/Argentina/ComodRivadavia" => "America/Argentina/Catamarca",
 	"America/Aruba" => "America/Curacao",
 	"America/Atka" => "America/Adak",
 	"America/Buenos_Aires" => "America/Argentina/Buenos_Aires",
 	"America/Catamarca" => "America/Argentina/Catamarca",
+	"America/Cayman" => "America/Panama",
 	"America/Coral_Harbour" => "America/Atikokan",
 	"America/Cordoba" => "America/Argentina/Cordoba",
 	"America/Dominica" => "America/Port_of_Spain",
@@ -312,9 +324,11 @@ my $li = q(+{
 	"America/Lower_Princes" => "America/Curacao",
 	"America/Marigot" => "America/Port_of_Spain",
 	"America/Mendoza" => "America/Argentina/Mendoza",
+	"America/Montreal" => "America/Toronto",
 	"America/Montserrat" => "America/Port_of_Spain",
 	"America/Porto_Acre" => "America/Rio_Branco",
 	"America/Rosario" => "America/Argentina/Cordoba",
+	"America/Santa_Isabel" => "America/Tijuana",
 	"America/Shiprock" => "America/Denver",
 	"America/St_Barthelemy" => "America/Port_of_Spain",
 	"America/St_Kitts" => "America/Port_of_Spain",
@@ -326,7 +340,9 @@ my $li = q(+{
 	"Antarctica/McMurdo" => "Pacific/Auckland",
 	"Antarctica/South_Pole" => "Pacific/Auckland",
 	"Arctic/Longyearbyen" => "Europe/Oslo",
+	"Asia/Aden" => "Asia/Riyadh",
 	"Asia/Ashkhabad" => "Asia/Ashgabat",
+	"Asia/Bahrain" => "Asia/Qatar",
 	"Asia/Calcutta" => "Asia/Kolkata",
 	"Asia/Chongqing" => "Asia/Shanghai",
 	"Asia/Chungking" => "Asia/Shanghai",
@@ -335,12 +351,17 @@ my $li = q(+{
 	"Asia/Istanbul" => "Europe/Istanbul",
 	"Asia/Kashgar" => "Asia/Urumqi",
 	"Asia/Katmandu" => "Asia/Kathmandu",
+	"Asia/Kuwait" => "Asia/Riyadh",
 	"Asia/Macao" => "Asia/Macau",
+	"Asia/Muscat" => "Asia/Dubai",
+	"Asia/Phnom_Penh" => "Asia/Bangkok",
+	"Asia/Rangoon" => "Asia/Yangon",
 	"Asia/Saigon" => "Asia/Ho_Chi_Minh",
 	"Asia/Tel_Aviv" => "Asia/Jerusalem",
 	"Asia/Thimbu" => "Asia/Thimphu",
 	"Asia/Ujung_Pandang" => "Asia/Makassar",
 	"Asia/Ulan_Bator" => "Asia/Ulaanbaatar",
+	"Asia/Vientiane" => "Asia/Bangkok",
 	"Atlantic/Faeroe" => "Atlantic/Faroe",
 	"Atlantic/Jan_Mayen" => "Europe/Oslo",
 	"Atlantic/St_Helena" => "Africa/Abidjan",
@@ -405,6 +426,9 @@ my $li = q(+{
 	Greenwich => "Etc/GMT",
 	Hongkong => "Asia/Hong_Kong",
 	Iceland => "Atlantic/Reykjavik",
+	"Indian/Antananarivo" => "Africa/Nairobi",
+	"Indian/Comoro" => "Africa/Nairobi",
+	"Indian/Mayotte" => "Africa/Nairobi",
 	Iran => "Asia/Tehran",
 	Israel => "Asia/Jerusalem",
 	Jamaica => "America/Jamaica",
@@ -419,7 +443,9 @@ my $li = q(+{
 	Navajo => "America/Denver",
 	PRC => "Asia/Shanghai",
 	"Pacific/Johnston" => "Pacific/Honolulu",
+	"Pacific/Midway" => "Pacific/Pago_Pago",
 	"Pacific/Ponape" => "Pacific/Pohnpei",
+	"Pacific/Saipan" => "Pacific/Guam",
 	"Pacific/Samoa" => "Pacific/Pago_Pago",
 	"Pacific/Truk" => "Pacific/Chuuk",
 	"Pacific/Yap" => "Pacific/Chuuk",
@@ -619,7 +645,7 @@ developed by Andrew Main (Zefram) <zefram@fysh.org>.
 The Olson timezone database is is the public domain.
 
 The C<Time::OlsonTZ::Data> Perl module wrapper for the database is
-Copyright (C) 2010, 2011, 2012, 2013, 2014
+Copyright (C) 2010, 2011, 2012, 2013, 2014, 2017
 Andrew Main (Zefram) <zefram@fysh.org>.
 
 =head1 LICENSE
